/* icode-common.h
 * Common part of instruction definition.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1997.
 */

#ifndef __icode_common_h
#define __icode_common_h

/*
 * These are for handling unknown types.
 */
#define	move_any			move_int
#define	move_anylong			move_long
#define	pusharg_any			pusharg_int
#define	pusharg_anylong			pusharg_long
#define	load_any			load_int
#define	load_anylong			load_long
#define	store_any			store_int
#define	store_anylong			store_long

/*
 * These are architecture specific.
 * References may be int's or long's depending on the machine.
 */
#if defined(HAVE_ref_eq_int)
#define	move_ref_const(DST, PTR)	move_int_const(DST, (int)(PTR))
#define	move_ref			move_int
#define	load_ref			load_int
#define	store_ref			store_int
#define	load_offset_ref			load_offset_int
#define	store_offset_ref		store_offset_int
#define	pusharg_ref			pusharg_int
#define	pusharg_ref_const(PTR, IDX)	pusharg_int_const((int)(PTR), (IDX))
#define	add_ref_const			add_int_const
#define	add_ref				add_int
#define	cmp_ref_const			cmp_int_const
#define	cmp_ref				cmp_int
#define	return_ref			return_int
#define	cbranch_ref_eq			cbranch_int_eq
#define	cbranch_ref_ne			cbranch_int_ne
#define	cbranch_ref_const_eq		cbranch_int_const_eq
#define	cbranch_ref_const_ne		cbranch_int_const_ne
#elif defined(HAVE_ref_eq_long)
#define	move_ref_const			move_long_const
#define	move_ref			move_long
#define	load_ref			load_long
#define	store_ref			store_long
#define	load_offset_ref			load_offset_long
#define	store_offset_ref		store_offset_long
#define	pusharg_ref			pusharg_long
#define	pusharg_ref_const(PTR, IDX)	pusharg_long_const((long)(PTR), (IDX))
#define	add_ref_const			add_long_const
#define	add_ref				add_long
#define	cmp_ref_const			cmp_long_const
#define	cmp_ref				cmp_long
#define	return_ref			return_long
#define	cbranch_ref_eq			cbranch_long_eq
#define	cbranch_ref_ne			cbranch_long_ne
#define	cbranch_ref_const_eq		cbranch_long_const_eq
#define	cbranch_ref_const_ne		cbranch_long_const_ne
#endif

/*
 * These are always the same.
 */
#define	returnarg_int()			(&returninfo[returnInt])
#define	returnarg_long()		(&returninfo[returnLong])
#define	returnarg_float()		(&returninfo[returnFloat])
#define	returnarg_double()		(&returninfo[returnDouble])
#define	returnarg_ref()			(&returninfo[returnRef])

#endif
