/* slots.c
 * Slots.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>
 */

#include "config.h"
#include "config-std.h"
#include "gtypes.h"
#include "slots.h"
#include "registers.h"
#include "md.h"

SlotInfo* basicslots;
SlotInfo* returninfo;
SlotInfo* slotinfo;
int maxslot;

SlotInfo* localinfo;
SlotInfo* stackinfo;
SlotInfo* tempinfo;

extern int maxLocal;

extern void* checked_calloc(size_t, size_t);


/*
 * Initiate slots.
 */
void
initSlots(int nrslots)
{
	int i;
	static int lastnrslots = 0;

	/* Allocate extra slots for return types and temps */
	nrslots += MAXRETURNS + MAXTEMPS;
	
	/* Make sure we have enough slots space */
	if (nrslots > lastnrslots) {
		if (basicslots != 0) {
			checked_free(basicslots);
		}
		basicslots = checked_calloc(nrslots, sizeof(SlotInfo));
		lastnrslots = nrslots;
	}
	/* Set 'maxslot' to the maximum slot usable (excluding returns) */
	maxslot = nrslots - MAXRETURNS;


        /* Free all slots */
        for (i = 0; i < nrslots; i++) {
		basicslots[i].insn = 0;
		basicslots[i].regno = NOREG;
		basicslots[i].modified = 0;
        }

	/* Setup various slot pointers */
	returninfo = &basicslots[0];
	slotinfo = &basicslots[MAXRETURNS];
	localinfo = 0;
	stackinfo = 0;
	tempinfo = 0;

	/* Setup return slots */
        returninfo[returnInt].regno = RETURN_INT;
        returninfo[returnRef].regno = RETURN_REF;
        returninfo[returnLong].regno = RETURN_LONG_LOW;
        returninfo[returnLong+1].regno = RETURN_LONG_HIGH;
        returninfo[returnFloat].regno = RETURN_FLOAT;
        returninfo[returnDouble].regno = RETURN_DOUBLE_LOW;
        returninfo[returnDouble+1].regno = RETURN_DOUBLE_HIGH;
}

/*
 * Mark return in use.
 */
void
markReturns(void)
{
        returninfo[returnInt].insn = 0;
        returninfo[returnRef].insn = 0;
        returninfo[returnLong].insn = 0;
        returninfo[returnLong+1].insn = 0;
        returninfo[returnFloat].insn = 0;
        returninfo[returnDouble].insn = 0;
        returninfo[returnDouble+1].insn = 0;
}
